# ZKM Kafka Connect Transforms


## Transforms 

### AddAdditionalPropertiesTransform

**IMPORTANT** This transform does not solve the problem for which it has been created !!! See https://git.zkm.de/data-infrastructure/infrastructure-doc/-/wikis/Kafka/Schema-Registries#json-schema-evolution-problems

**DO NOT USE!!!**

A custom Single Message Transform (SMT) for Kafka Connect that automatically adds `"additionalProperties": true` to JSON schemas to handle evolving database schemas in CDC/ETL processes.

#### Problem

When using JSON Schema with Kafka Schema Registry in BACKWARDS compatibility mode, adding new fields to database tables can cause schema registration errors, even though JSON Schema normally allows additional properties by default:

```
io.confluent.kafka.schemaregistry.rest.exceptions.RestIncompatibleSchemaException: Schema being registered is incompatible with an earlier schema for subject "private.cdc.zkm-archive.01.Personen_Archiv-value", details: [Found incompatible change: Difference{jsonPath='#/properties/Hilfsfeld_Person_Lebensdaten_Formel', type=PROPERTY_ADDED_TO_OPEN_CONTENT_MODEL}]
```

This transform is a misguided attempt to solve the problem by explicitly adding `"additionalProperties": true` to all JSON schemas produced by Kafka Connect.

#### Features

- Automatically modifies JSON schemas to explicitly allow additional properties
- Preserves all existing schema structure and properties

#### Usage

Add the transform to your connector configuration:

```properties
transforms=addAdditionalProperties
transforms.addAdditionalProperties.type=de.zkm.kafka.connect.transforms.AddAdditionalPropertiesTransform
```

#### Alternative Approaches

If you prefer not to use the custom SMT, you can also:

1. Set `value.converter.json.fail.unknown.properties=false` in your connector config to make the deserializer more lenient
2. Use the Schema Registry REST API to set compatibility to NONE for specific subjects
3. Manually update schemas through the Schema Registry API to include `"additionalProperties": true`

#### How It Works

The transform inspects each record passing through Kafka Connect and modifies its schema by adding the `additionalProperties` parameter. This ensures that when the schema is registered with Schema Registry, it will explicitly allow additional properties, resolving the compatibility error.

## Getting Started

### Prerequisites

- Apache Kafka
- Confluent Schema Registry
- Kafka Connect

### Installation

1. Build the JAR file:

```bash
mvn clean package
```

2. Download and install the resulting `kafka-connect-transforms-1.0.0.zip` as Kafka Connect plugin in your Dockerfile. For example:

```bash
echo "installing kafka-connect-transforms-1.0.0" && \
curl -L -o /tmp/kafka-connect-transforms-1.0.0.zip https://reposilite.mutech.zkm.de/releases/de/zkm/kafka-connect-transforms/1.0.0/kafka-connect-transforms-1.0.0.zip
 && \
confluent-hub install --no-prompt /tmp/kafka-connect-transforms-1.0.0.zip & \
```

3. Restart Kafka Connect to pick up the new plugin.

## Building

```bash
mvn clean package
```

## License

This project is licensed under the MIT License

## Contributing

Contributions are welcome! Please feel free to submit a Pull Request.